﻿using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Input;
using System.Runtime.InteropServices;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using System.Collections.ObjectModel;
using System.Threading;
using WPF_Wii_Binding_Library;


namespace WPF_Wii_Visualizer
{
    public partial class Window1
	{
        WPF_Wii holder;

        Mutex mu = new Mutex();

        Point LastLeftPoint = new Point();
        Grid LeftHandActive = new Grid();

        ObservableCollection<IRGrid> IRPoints = new ObservableCollection<IRGrid>();
        ObservableCollection<IRGrid> LeftPoints = new ObservableCollection<IRGrid>();
        ObservableCollection<IRGrid> RightPoints = new ObservableCollection<IRGrid>();
        
        ObservableCollection<MPGrid> MultiPointObjects = new ObservableCollection<MPGrid>();
        ObservableCollection<MPGrid> LeftPinchedObjects = new ObservableCollection<MPGrid>();
        ObservableCollection<MPGrid> RightPinchedObjects = new ObservableCollection<MPGrid>();

        bool isLeftPinched;
        bool isRightPinched;

        Point iPinchedDistance;
        Point iMidPoint;
        
        MPGrid LeftPinchedGrid;
        MPGrid RightPinchedGrid;

        IRGrid LeftPinchedIRPoint;
        IRGrid RightPinchedIRPoint;

        UIElementCollection MPCollection ;

        double GridScaleHolder = 1;

        int largestZ;

        public Window1()
		{
			this.InitializeComponent();

            holder = (WPF_Wii)this.Resources["WiiData"];

            CompositionTarget.Rendering += new EventHandler(StoreIRData);
            CompositionTarget.Rendering += new EventHandler(MoveItem);

            IRPoints.Add(VisiblePoint1);
            IRPoints.Add(VisiblePoint2);
            IRPoints.Add(VisiblePoint3);
            IRPoints.Add(VisiblePoint4);           

            isLeftPinched = false;
            isRightPinched = false;
            MPCollection = MPItemsCanvas.Children;

            /*foreach (MPGrid MPItem in MPCollection)
            {
                MultiPointObjects.Add(MPItem);
                MPItem.CurrentPosition = new Point(Canvas.GetLeft(MPItem), Canvas.GetTop(MPItem));
                MPItem.iDimensions = new Point(MPItem.ActualWidth, MPItem.ActualHeight);
            }*/

            largestZ = MultiPointObjects.Count;
		}

        private void StopWiimoteBtn_Click(object sender, RoutedEventArgs e)
        {
            holder.stopWiiReading();
        }

        private void startWiimote_Click(object sender, RoutedEventArgs e)
        {
            holder.startWiiReading();

            foreach (MPGrid MPItem in MPCollection)
            {
                MultiPointObjects.Add(MPItem);
                MPItem.CurrentPosition = new Point(Canvas.GetLeft(MPItem), Canvas.GetTop(MPItem));
                MPItem.iDimensions = new Point(MPItem.ActualWidth, MPItem.ActualHeight);
            }
        }        

        private void SaveIRData(object sender, RoutedEventArgs e)
        {
        }

        private void TwoPointHitTest(object LED1, object LED2, object[] CanvasObjects, RoutedEventArgs e)
        {

        }

        private void IRVisibilityChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            IRGrid VisHolder = sender as IRGrid;

            //Visibility newVisibility = (Visibility)e.NewValue;
            bool newValueHolder = (bool)e.NewValue;

            if (newValueHolder)
            {
                assignLeftRight(VisHolder);

                if (isLeftPinched && isRightPinched)
                {
                    LeftPinchedGrid.iScale = GridScaleHolder;
                }

                foreach (IRGrid switchPinch in IRPoints)
                {
                    if (switchPinch.Name != VisHolder.Name)
                    {
                        isLeftPinched = false;
                        //LPinched.Text = "false";
                        foreach (MPGrid unPinch in MultiPointObjects)
                        {
                            unPinch.IsLeftPinched = false;
                        }
                    }

                    if (switchPinch.Name != VisHolder.Name)
                    {
                        isRightPinched = false;
                        //RPinched.Text = "false";
                        foreach (MPGrid unPinch in MultiPointObjects)
                        {
                            unPinch.IsRightPinched = false;
                        }
                    }
                }

            }
            else
            {
                Point tempPoint1 = new Point();
                Point tempPoint2 = new Point();

                double point12Distance;
                double CanvasPixels = IRCanvas.ActualHeight * IRCanvas.ActualWidth;
                
                if (VisHolder.IsLeftHand && 
                    (LeftPoints.Count == 2) &&
                    (LeftPoints[0] != null) )
                {
                    IRGrid RemainingGrid = new IRGrid(); 
                    
                    if (VisHolder == LeftPoints[0])
                    {
                        RemainingGrid = LeftPoints[1];
                    }
                    else if (VisHolder == LeftPoints[1])
                    {
                        RemainingGrid = LeftPoints[0];
                    }                    

                    tempPoint1.X = Canvas.GetLeft(LeftPoints[0]);
                    tempPoint1.Y = Canvas.GetTop(LeftPoints[0]);
                    tempPoint2.X = Canvas.GetLeft(LeftPoints[1]);
                    tempPoint2.Y = Canvas.GetTop(LeftPoints[1]);

                    point12Distance = Math.Sqrt(Math.Pow((tempPoint1.X - tempPoint2.X), 2) +
                                                   Math.Pow((tempPoint1.Y - tempPoint2.Y), 2));
                    //LPinched.Text = "true";

                    if (point12Distance < (CanvasPixels / 12000))
                    {
                        //Storyboard runLeftStoryboard = this.FindResource("LeftRegister") as Storyboard;
                        //this.BeginStoryboard(runLeftStoryboard);
                        isLeftPinched = true;
                        RemainingGrid.IsPinched = true;
                        if (isRightPinched)
                        {
                            iPinchedDistance.X = Canvas.GetLeft(RightPinchedIRPoint) - Canvas.GetLeft(RemainingGrid);
                            iPinchedDistance.Y = Canvas.GetTop(RightPinchedIRPoint) - Canvas.GetTop(RemainingGrid);
                            iMidPoint.X = Canvas.GetLeft(RemainingGrid) + (iPinchedDistance.X * .5);
                            iMidPoint.Y = Canvas.GetTop(RemainingGrid) + (iPinchedDistance.Y * .5);
                        }


                        // then, for each item in the larger observable collection
                        // do hit point testing on each one.

                        LeftPinchedObjects.Clear();

                        foreach (MPGrid LeftHitGrid in MultiPointObjects)
                        {
                            if (LeftHitGrid.hitTest(RemainingGrid, LeftHitGrid))
                            {
                                LeftPinchedObjects.Add(LeftHitGrid);
                            }
                        }

                        // zindex testing to see what is selected and flip the 
                        // left pinched boolean on it. 
                        if (LeftPinchedObjects.Count > 0)
                        {
                            MPGrid FinalGrid = LeftPinchedObjects[0];
                            int tempZIndex = 0;
                            foreach (MPGrid lastCheck in LeftPinchedObjects)
                            {
                                if (Panel.GetZIndex(lastCheck) > tempZIndex)
                                {
                                    tempZIndex = Panel.GetZIndex(lastCheck);
                                    FinalGrid = lastCheck;
                                }
                            }

                            FinalGrid.IsLeftPinched = true;
                            largestZ++;
                            Panel.SetZIndex(FinalGrid, largestZ);

                            //save these grids so I don't have to find them again. 
                            LeftPinchedGrid = FinalGrid;
                            LeftPinchedGrid.iLocation = new Point(Canvas.GetLeft(LeftPinchedGrid),
                                                                  Canvas.GetTop(LeftPinchedGrid));

                        }
                        LeftPinchedIRPoint = RemainingGrid;
                    }

                    
                }
                else if (VisHolder.IsRightHand &&
                    (RightPoints.Count == 2) &&
                    (RightPoints[0] != null))
                {
                    IRGrid RemainingGrid = new IRGrid();

                    if (VisHolder == RightPoints[0])
                    {
                        RemainingGrid = RightPoints[1];
                    }
                    else if (VisHolder == RightPoints[1])
                    {
                        RemainingGrid = RightPoints[0];
                    }

                    tempPoint1.X = Canvas.GetLeft(RightPoints[0]);
                    tempPoint1.Y = Canvas.GetTop(RightPoints[0]);
                    tempPoint2.X = Canvas.GetLeft(RightPoints[1]);
                    tempPoint2.Y = Canvas.GetTop(RightPoints[1]);

                    point12Distance = Math.Sqrt(Math.Pow((tempPoint1.X - tempPoint2.X), 2) +
                                                   Math.Pow((tempPoint1.Y - tempPoint2.Y), 2));
                    //RPinched.Text = "true";

                    if (point12Distance < (CanvasPixels / 12000))
                    {
                        //Storyboard runLeftStoryboard = this.FindResource("RightRegister") as Storyboard;
                        //this.BeginStoryboard(runLeftStoryboard);
                        isRightPinched = true;
                        RemainingGrid.IsPinched = true;
                        if (isLeftPinched)
                        {
                            iPinchedDistance.X = Canvas.GetLeft(RemainingGrid) - Canvas.GetLeft(LeftPinchedIRPoint);
                            iPinchedDistance.Y = Canvas.GetTop(RemainingGrid) - Canvas.GetTop(LeftPinchedIRPoint) ;
                            iMidPoint.X = Canvas.GetLeft(LeftPinchedIRPoint) + (iPinchedDistance.X * .5);
                            iMidPoint.Y = Canvas.GetTop(LeftPinchedIRPoint) + (iPinchedDistance.Y * .5);
                        }
                    }

                    // then, for each item in the larger observable collection
                    // do hit point testing on each one.

                    RightPinchedObjects.Clear();

                    foreach (MPGrid RightHitGrid in MultiPointObjects)
                    {
                        if (RightHitGrid.hitTest(RemainingGrid, RightHitGrid))
                        {
                            RightPinchedObjects.Add(RightHitGrid);
                        }
                    }

                    // zindex testing to see what is selected and flip the 
                    // left pinched boolean on it. 
                    if (RightPinchedObjects.Count > 0)
                    {
                        MPGrid FinalGrid = RightPinchedObjects[0];
                        int tempZIndex = -2;
                        foreach (MPGrid lastCheck in RightPinchedObjects)
                        {
                            if (Panel.GetZIndex(lastCheck) > tempZIndex)
                            {
                                tempZIndex = Panel.GetZIndex(lastCheck);
                                FinalGrid = lastCheck;
                            }
                        }

                        FinalGrid.IsRightPinched = true;
                        largestZ++;
                        Panel.SetZIndex(FinalGrid, largestZ);

                        //save these grids so I don't have to find them again. 
                        RightPinchedGrid = FinalGrid;
                        RightPinchedGrid.iLocation = new Point(Canvas.GetLeft(RightPinchedGrid),
                                                               Canvas.GetTop(RightPinchedGrid));

                    }
                    RightPinchedIRPoint = RemainingGrid;                   
                }

                VisHolder.IsRightHand = false;
                VisHolder.IsLeftHand = false;
            }
        }

        void assignLeftRight(IRGrid NewGrid){
            
            int LeftCount = 0;
            int RightCount = 0;
            int xRank = 0; //4 is rightmost, 0 is leftmost
            double[] xPositions = new double[4];
            int newGridIndex = 0;
            Point surrogatePoint = new Point(0,0);
            LeftPoints.Clear();
            RightPoints.Clear();
            ObservableCollection<IRGrid> VisiblePoints = new ObservableCollection<IRGrid>();
            IRGrid[] trialArray = new IRGrid[4];

            if (NewGrid.Name == VisiblePoint1.Name)
            {
                surrogatePoint.X = (holder.IR1Position.X * IRCanvas.ActualWidth);
                surrogatePoint.Y = (holder.IR1Position.Y * IRCanvas.ActualHeight);
                NewGrid.CurrentPosition = surrogatePoint;
            }
            else if (NewGrid.Name == VisiblePoint2.Name)
            {
                surrogatePoint.X = (holder.IR2Position.X * IRCanvas.ActualWidth);
                surrogatePoint.Y = (holder.IR2Position.Y * IRCanvas.ActualHeight);
                NewGrid.CurrentPosition = surrogatePoint;
            } 
            else if (NewGrid.Name == VisiblePoint3.Name)
            {
                surrogatePoint.X = (holder.IR3Position.X * IRCanvas.ActualWidth);
                surrogatePoint.Y = (holder.IR3Position.Y * IRCanvas.ActualHeight);
                NewGrid.CurrentPosition = surrogatePoint;
            }
            else if (NewGrid.Name == VisiblePoint4.Name)
            {
                surrogatePoint.X = (holder.IR4Position.X * IRCanvas.ActualWidth);
                surrogatePoint.Y = (holder.IR4Position.Y * IRCanvas.ActualHeight);
                NewGrid.CurrentPosition = surrogatePoint;
            }
            
            foreach (IRGrid IRPoint in IRPoints)
            {
                if (IRPoint.Visibility == Visibility.Visible)
                {
                    if (IRPoint == NewGrid)
                    {
                        newGridIndex = VisiblePoints.Count;
                        xPositions[VisiblePoints.Count] = NewGrid.CurrentPosition.X;
                    }
                    else
                    {
                        xPositions[VisiblePoints.Count] = Canvas.GetLeft(IRPoint);
                    }
                    trialArray[VisiblePoints.Count] = IRPoint;

                    if (Canvas.GetLeft(IRPoint) < NewGrid.CurrentPosition.X)
                    {
                        xRank++;
                    }

                    if (IRPoint == NewGrid)
                    {
                        newGridIndex = VisiblePoints.Count;
                    }

                    VisiblePoints.Add(IRPoint);                    
                }
            }

            if (VisiblePoints.Count == 1)
            {
                NewGrid.IsRightHand = false;
                NewGrid.IsLeftHand = true;
                LeftPoints.Add(NewGrid);
            }

            if (VisiblePoints.Count == 2)
            {
                Array.Sort(xPositions, trialArray);
 
                double xDistance = Math.Abs(xPositions[0] - xPositions[1]);

                if (xDistance < 300)
                {
                    trialArray[2].IsLeftHand = true;
                    trialArray[2].IsRightHand = false;
                    trialArray[3].IsLeftHand = true;
                    trialArray[3].IsRightHand = false;
                    LeftPoints.Add(trialArray[2]);
                    LeftPoints.Add(trialArray[3]);
                }
                else
                {
                    trialArray[2].IsLeftHand = true;
                    trialArray[2].IsRightHand = false;
                    trialArray[3].IsLeftHand = false;
                    trialArray[3].IsRightHand = true;
                    LeftPoints.Add(trialArray[2]);
                    RightPoints.Add(trialArray[3]);
                }
            }

            if (VisiblePoints.Count == 3)
            {
                Array.Sort(xPositions, trialArray);
                
                trialArray[1].IsLeftHand = true;
                trialArray[1].IsRightHand = false;
                LeftPoints.Add(trialArray[1]);

                trialArray[3].IsLeftHand = false;
                trialArray[3].IsRightHand = true;
                RightPoints.Add(trialArray[3]);

                double oneTwoDifference;
                double twoThreeDifference;
                oneTwoDifference = Math.Abs(xPositions[0] - xPositions[1]);
                twoThreeDifference = Math.Abs(xPositions[1] - xPositions[2]);

                if (oneTwoDifference > twoThreeDifference)
                {
                    trialArray[2].IsLeftHand = false;
                    trialArray[2].IsRightHand = true;
                    RightPoints.Add(trialArray[2]);
                }
                else
                {
                    trialArray[2].IsLeftHand = true;
                    trialArray[2].IsRightHand = false;
                    LeftPoints.Add(trialArray[2]);
                }
            }

            if (VisiblePoints.Count == 4)
            {
                Array.Sort(xPositions, trialArray);
                trialArray[0].IsLeftHand = true;
                trialArray[0].IsRightHand = false;
                LeftPoints.Add(trialArray[0]);
                
                trialArray[1].IsLeftHand = true;
                trialArray[1].IsRightHand = false;
                LeftPoints.Add(trialArray[1]);
                
                trialArray[2].IsLeftHand = false;
                trialArray[2].IsRightHand = true;
                RightPoints.Add(trialArray[2]);
                
                trialArray[3].IsLeftHand = false;
                trialArray[3].IsRightHand = true;
                RightPoints.Add(trialArray[3]);
            }

        }

        void StoreIRData(object sender, EventArgs e)
        {
            Point tempPoint = new Point(0, 0);
            Point velocityPoint = new Point(0,0);

            if (VisiblePoint1.Visibility == Visibility.Visible)
            {
                tempPoint.X = Canvas.GetLeft(VisiblePoint1);
                tempPoint.Y = Canvas.GetTop(VisiblePoint1);
                VisiblePoint1.LastPosition = VisiblePoint1.CurrentPosition;
                VisiblePoint1.CurrentPosition = tempPoint;
                velocityPoint.X = VisiblePoint1.CurrentPosition.X - VisiblePoint1.LastPosition.X;
                velocityPoint.Y = VisiblePoint1.CurrentPosition.Y - VisiblePoint1.LastPosition.Y;
                VisiblePoint1.Velocity = velocityPoint;
            }

            if (VisiblePoint2.Visibility == Visibility.Visible)
            {
                tempPoint.X = Canvas.GetLeft(VisiblePoint2);
                tempPoint.Y = Canvas.GetTop(VisiblePoint2);
                VisiblePoint2.LastPosition = VisiblePoint2.CurrentPosition;
                VisiblePoint2.CurrentPosition = tempPoint;
                velocityPoint.X = VisiblePoint2.CurrentPosition.X - VisiblePoint2.LastPosition.X;
                velocityPoint.Y = VisiblePoint2.CurrentPosition.Y - VisiblePoint2.LastPosition.Y;
                VisiblePoint2.Velocity = velocityPoint;
            }

            if (VisiblePoint3.Visibility == Visibility.Visible)
            {
                tempPoint.X = Canvas.GetLeft(VisiblePoint3);
                tempPoint.Y = Canvas.GetTop(VisiblePoint3);
                VisiblePoint3.LastPosition = VisiblePoint3.CurrentPosition;
                VisiblePoint3.CurrentPosition = tempPoint;
                velocityPoint.X = VisiblePoint3.CurrentPosition.X - VisiblePoint3.LastPosition.X;
                velocityPoint.Y = VisiblePoint3.CurrentPosition.Y - VisiblePoint3.LastPosition.Y;
                VisiblePoint3.Velocity = velocityPoint;
            }

            if (VisiblePoint4.Visibility == Visibility.Visible)
            {
                tempPoint.X = Canvas.GetLeft(VisiblePoint4);
                tempPoint.Y = Canvas.GetTop(VisiblePoint4);
                VisiblePoint4.LastPosition = VisiblePoint4.CurrentPosition;
                VisiblePoint4.CurrentPosition = tempPoint;
                velocityPoint.X = VisiblePoint4.CurrentPosition.X - VisiblePoint4.LastPosition.X;
                velocityPoint.Y = VisiblePoint4.CurrentPosition.Y - VisiblePoint4.LastPosition.Y;
                VisiblePoint4.Velocity = velocityPoint;
            }
        }

        void MoveItem(object sender, EventArgs e)
        {

            if ((isLeftPinched && isRightPinched) && 
                (RightPinchedGrid == LeftPinchedGrid) &&
                (RightPinchedGrid != null))
            {
                MPGrid MPHolder = RightPinchedGrid;

                Point cPinchedDistance = new Point((Canvas.GetLeft(RightPinchedIRPoint) - Canvas.GetLeft(LeftPinchedIRPoint)),
                                                    (Canvas.GetTop(RightPinchedIRPoint) - Canvas.GetTop(LeftPinchedIRPoint)));
                //Point IRMidVelocity = new Point((RightPinchedIRPoint.Velocity.X - LeftPinchedIRPoint.Velocity.X),
                //                                    (RightPinchedIRPoint.Velocity.Y - LeftPinchedIRPoint.Velocity.Y));
                double cIRDistance = Math.Sqrt(Math.Pow(cPinchedDistance.X, 2) +
                                                     Math.Pow(cPinchedDistance.Y, 2));
                double iIRDistance = Math.Sqrt(Math.Pow(iPinchedDistance.X, 2) +
                                                    Math.Pow(iPinchedDistance.Y, 2));
                Point cMidPoint = new Point((Canvas.GetLeft(LeftPinchedIRPoint) + (cPinchedDistance.X * .5)),
                                            (Canvas.GetTop(LeftPinchedIRPoint) + (cPinchedDistance.Y * .5)));
                
                double scaleX = cPinchedDistance.X / iPinchedDistance.X;
                double scaleY = cPinchedDistance.Y / iPinchedDistance.Y;
                double distanceRatio = cIRDistance / iIRDistance;
                
                distanceRatio = MPHolder.iScale * distanceRatio;

                if (distanceRatio > 5)
                {
                    distanceRatio = 5;
                }
                
                /*Point scaleComp = new Point();
                scaleComp.X = ((1 - distanceRatio) * MPHolder.iDimensions.X) / 2;
                scaleComp.Y = ((1 - distanceRatio) * MPHolder.iDimensions.Y) / 2;*/

                if (MPHolder.CanScale)
                {
                    Point scaleComp = new Point();
                    scaleComp.X = ((1 - distanceRatio) * MPHolder.iDimensions.X) / 2;
                    scaleComp.Y = ((1 - distanceRatio) * MPHolder.iDimensions.Y) / 2; 
                    
                    Canvas.SetLeft(MPHolder, (MPHolder.iLocation.X + (cMidPoint.X - iMidPoint.X) + scaleComp.X));
                    Canvas.SetTop(MPHolder, (MPHolder.iLocation.Y + (cMidPoint.Y - iMidPoint.Y) + scaleComp.Y));

                    //distanceRatio = MPHolder.iScale * distanceRatio;
                    
                    ScaleTransform gridScaling = new ScaleTransform(distanceRatio, distanceRatio);
                    TransformGroup gridTransform = new TransformGroup();
                    
                    gridTransform.Children.Add(gridScaling);
                    MPHolder.LayoutTransform = gridTransform;
                    GridScaleHolder = distanceRatio;                   
                }
                else
                {
                    Point scaleComp = new Point();
                    scaleComp.X = ((1 - distanceRatio) * MPHolder.iDimensions.X) / 2;
                    scaleComp.Y = ((1 - distanceRatio) * MPHolder.iDimensions.Y) / 2;

                    Canvas.SetLeft(MPHolder, (MPHolder.iLocation.X + (cMidPoint.X - iMidPoint.X) + scaleComp.X));
                    Canvas.SetTop(MPHolder, (MPHolder.iLocation.Y + (cMidPoint.Y - iMidPoint.Y) + scaleComp.Y)); 
                    

                    MPHolder.Width = MPHolder.iDimensions.X * distanceRatio;
                    MPHolder.Height = MPHolder.iDimensions.Y * distanceRatio;


                }
            }

            else
            {

                if (isLeftPinched &&
                    LeftPinchedObjects.Count > 0)
                {
                    Point newVelocity = new Point();

                    //track the changes of the IRGrid, and
                    //assign those changes to the MPGrid
                    Point IRdifference = LeftPinchedIRPoint.Velocity;
                    Point newLeftPinchedPosition = new Point();
                    newLeftPinchedPosition.X = Canvas.GetLeft(LeftPinchedGrid) + IRdifference.X;
                    newLeftPinchedPosition.Y = Canvas.GetTop(LeftPinchedGrid)+ IRdifference.Y;
                    Canvas.SetLeft(LeftPinchedGrid, newLeftPinchedPosition.X);
                    Canvas.SetTop(LeftPinchedGrid, newLeftPinchedPosition.Y);

                    LeftPinchedGrid.LastPosition = LeftPinchedGrid.CurrentPosition;
                    LeftPinchedGrid.CurrentPosition = newLeftPinchedPosition;
                    newVelocity.X = Canvas.GetLeft(LeftPinchedGrid) - LeftPinchedGrid.LastPosition.X;
                    newVelocity.Y = Canvas.GetTop(LeftPinchedGrid) - LeftPinchedGrid.LastPosition.Y;
                    LeftPinchedGrid.Velocity = newVelocity;


                }

                if (isRightPinched &&
                        RightPinchedObjects.Count > 0)
                {
                    Point newVelocity = new Point();

                    //track the changes of the IRGrid, and
                    //assign those changes to the MPGrid
                    Point IRdifference = RightPinchedIRPoint.Velocity;
                    Point newRightPinchedPosition = new Point();
                    //newRightPinchedPosition.X = RightPinchedGrid.CurrentPosition.X + IRdifference.X;
                    //newRightPinchedPosition.Y = RightPinchedGrid.CurrentPosition.Y + IRdifference.Y;
                    newRightPinchedPosition.X = Canvas.GetLeft(RightPinchedGrid) + IRdifference.X;
                    newRightPinchedPosition.Y = Canvas.GetTop(RightPinchedGrid) + IRdifference.Y;
                    Canvas.SetLeft(RightPinchedGrid, newRightPinchedPosition.X);
                    Canvas.SetTop(RightPinchedGrid, newRightPinchedPosition.Y);

                    RightPinchedGrid.LastPosition = RightPinchedGrid.CurrentPosition;
                    RightPinchedGrid.CurrentPosition = newRightPinchedPosition;
                    newVelocity.X = (RightPinchedGrid.CurrentPosition.X - RightPinchedGrid.LastPosition.X);
                    newVelocity.Y = (RightPinchedGrid.CurrentPosition.Y - RightPinchedGrid.LastPosition.Y);
                    RightPinchedGrid.Velocity = newVelocity;
                }
            }
            
            foreach (MPGrid MovementGrid in MultiPointObjects)
            {
                
                if ((MovementGrid.Velocity.X != 0.0) && 
                    (MovementGrid.Velocity.Y != 0.0) &&
                    (!MovementGrid.IsLeftPinched) && 
                    (!MovementGrid.IsRightPinched))
                {
                    Point setNewPoint = MovementGrid.updatePosition(MovementGrid,
                                                                    new Point(MPItemsCanvas.ActualWidth, MPItemsCanvas.ActualHeight));
                    MovementGrid.CurrentPosition = setNewPoint;
                    //Canvas.SetLeft(MovementGrid, setNewPoint.X);
                    //Canvas.SetTop(MovementGrid, setNewPoint.Y);
                }
            }
        }

        private void LayoutMPGrid_Initialized(object sender, EventArgs e)
        {
            /*foreach (MPGrid MPItem in MPCollection)
            {
                MultiPointObjects.Add(MPItem);
                MPItem.CurrentPosition = new Point(Canvas.GetLeft(MPItem), Canvas.GetTop(MPItem));
                MPItem.iDimensions = new Point(MPItem.ActualWidth, MPItem.ActualHeight);
            } */
        }

        private void ResetObjects(object sender, RoutedEventArgs e)
        {
             Canvas.SetLeft(PointImage, 100);
            Canvas.SetTop(PointImage, 50);
            Canvas.SetLeft(PointImage2, 400);
            Canvas.SetTop(PointImage2, 300);
            Canvas.SetLeft(PointImage3, 500);
            Canvas.SetTop(PointImage3, 50);

            ScaleTransform gridScaling = new ScaleTransform(1, 1);
            TransformGroup gridTransform = new TransformGroup();

            gridTransform.Children.Add(gridScaling);
            PointImage.LayoutTransform = gridTransform;
            PointImage2.LayoutTransform = gridTransform;
            PointImage3.LayoutTransform = gridTransform;
            PointImage.iScale = 1;
            PointImage2.iScale = 1;
            PointImage3.iScale = 1;
        }
    }
        
        public class MPGrid : Grid
    {
        static MPGrid()
        {
            MPGrid.VelocityProperty = DependencyProperty.Register("Velocity",
                        typeof(Point), typeof(MPGrid),
                        new FrameworkPropertyMetadata(new Point(0,0)));

            MPGrid.LastPositionProperty = DependencyProperty.Register("LastPosition",
                        typeof(Point), typeof(MPGrid),
                        new FrameworkPropertyMetadata(new Point(0, 0)));

            MPGrid.CurrentPositionProperty = DependencyProperty.Register("CurrentPosition",
                        typeof(Point), typeof(MPGrid),
                        new FrameworkPropertyMetadata(new Point(0, 0), new PropertyChangedCallback(onCurrentPositionChanged)));

            MPGrid.FrictionProperty = DependencyProperty.Register("Friction",
                        typeof(double), typeof(MPGrid),
                        new FrameworkPropertyMetadata(0.2));

            MPGrid.IsLeftPinchedProperty = DependencyProperty.Register("IsLeftPinched",
                        typeof(bool), typeof(MPGrid),
                        new FrameworkPropertyMetadata(false));

            MPGrid.IsRightPinchedProperty = DependencyProperty.Register("IsRightPinched",
                        typeof(bool), typeof(MPGrid),
                        new FrameworkPropertyMetadata(false));
            
            MPGrid.CanScaleProperty = DependencyProperty.Register("CanScale",
                        typeof(bool), typeof(MPGrid),
                        new FrameworkPropertyMetadata(true));

            MPGrid.iScaleProperty = DependencyProperty.Register("iScale",
                        typeof(double), typeof(MPGrid),
                        new FrameworkPropertyMetadata(1.0));

            MPGrid.iAngleProperty = DependencyProperty.Register("iAngle",
                        typeof(double), typeof(MPGrid),
                        new FrameworkPropertyMetadata(0.0));

            MPGrid.iDimensionsProperty = DependencyProperty.Register("iDimensions",
                        typeof(Point), typeof(MPGrid),
                        new FrameworkPropertyMetadata(new Point()));

            MPGrid.iLocationProperty = DependencyProperty.Register("iLocation",
                        typeof(Point), typeof(MPGrid),
                        new FrameworkPropertyMetadata(new Point()));
        }

        public Point Velocity
        {
            get { return (Point)GetValue(MPGrid.VelocityProperty); }
            set { SetValue(MPGrid.VelocityProperty, value); }
        }

        public Point LastPosition
        {
            get { return (Point)GetValue(MPGrid.LastPositionProperty); }
            set { SetValue(MPGrid.LastPositionProperty, value); }
        }

        public Point CurrentPosition
        {
            get { return (Point)GetValue(MPGrid.CurrentPositionProperty); }
            set { SetValue(MPGrid.CurrentPositionProperty, value); }
        }

        public double Friction
        {
            get { return (double)GetValue(MPGrid.FrictionProperty); }
            set { SetValue(MPGrid.FrictionProperty, value); }
        }

        public bool IsLeftPinched
        {
            get { return (bool)GetValue(MPGrid.IsLeftPinchedProperty); }
            set { SetValue(MPGrid.IsLeftPinchedProperty, value); }
        }

        public bool IsRightPinched
        {
            get { return (bool)GetValue(MPGrid.IsRightPinchedProperty); }
            set { SetValue(MPGrid.IsRightPinchedProperty, value); }
        }

        public bool CanScale
        {
            get { return (bool)GetValue(MPGrid.CanScaleProperty); }
            set { SetValue(MPGrid.CanScaleProperty, value); }
        }
                
        public double iScale
        {
            get { return (double)GetValue(MPGrid.iScaleProperty); }
            set { SetValue(MPGrid.iScaleProperty, value); }
        }

        public Point iDimensions
        {
            get { return (Point)GetValue(MPGrid.iDimensionsProperty); }
            set { SetValue(MPGrid.iDimensionsProperty, value); }
        }

        public Point iLocation
        {
            get { return (Point)GetValue(MPGrid.iLocationProperty); }
            set { SetValue(MPGrid.iLocationProperty, value); }
        }

        public double iAngle
        {
            get { return (double)GetValue(MPGrid.iAngleProperty); }
            set { SetValue(MPGrid.iAngleProperty, value); }
        }

        private static void onLastPointChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
                        
        }

        private static void onCurrentPositionChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {

        }

        public static DependencyProperty VelocityProperty;
        public static DependencyProperty LastPositionProperty;
        public static DependencyProperty CurrentPositionProperty;
        public static DependencyProperty FrictionProperty;
        public static DependencyProperty IsLeftPinchedProperty;
        public static DependencyProperty IsRightPinchedProperty;
        public static DependencyProperty CanScaleProperty;
        
        public static DependencyProperty iScaleProperty;
        public static DependencyProperty iAngleProperty;
        public static DependencyProperty iDimensionsProperty;
        public static DependencyProperty iLocationProperty;

        public Point updatePosition(object targetGrid, Point boundPoint)
        {
            MPGrid holder = targetGrid as MPGrid;

            Point newPoint = new Point(0, 0);
            Point velocityPoint = (Point)holder.Velocity;
            double frictionRatioX;
            double frictionRatioY;            

            holder.LastPosition = holder.CurrentPosition;
            frictionRatioX = holder.Velocity.X /(Math.Sqrt(Math.Pow(holder.Velocity.X, 2) + Math.Pow(holder.Velocity.Y, 2)));
            frictionRatioY = holder.Velocity.Y /(Math.Sqrt(Math.Pow(holder.Velocity.X, 2) + Math.Pow(holder.Velocity.Y, 2)));

            if (velocityPoint.X < 0)
            {
                velocityPoint.X = velocityPoint.X + (holder.Friction * frictionRatioX);
            }
            else if (Math.Abs(velocityPoint.X) < holder.Friction)
            {
                velocityPoint.X = 0;
            }
            else if (velocityPoint.X > 0)
            {
                velocityPoint.X = velocityPoint.X - (holder.Friction * frictionRatioX);
            }

            if (velocityPoint.Y < 0)
            {
                velocityPoint.Y = velocityPoint.Y + (holder.Friction * frictionRatioY);
            }
            else if (Math.Abs(velocityPoint.Y) < holder.Friction)
            {
                velocityPoint.Y = 0;
            }
            else if (velocityPoint.Y > 0)
            {
                velocityPoint.Y = velocityPoint.Y - (holder.Friction * frictionRatioY);
            }

            newPoint.X = holder.CurrentPosition.X + velocityPoint.X;
            newPoint.Y = holder.CurrentPosition.Y + velocityPoint.Y;

            if (newPoint.X < 0 || (newPoint.X + holder.ActualWidth) > boundPoint.X)
            {
                velocityPoint.X = -(velocityPoint.X);
            }
            if (newPoint.Y < 0 || (newPoint.Y + holder.ActualHeight) > boundPoint.Y)
            {
                velocityPoint.Y = -(velocityPoint.Y);
            }

            holder.Velocity = velocityPoint;

            return newPoint;
        }

        public bool hitTest(object IRObject, object HitObject)
        {
            IRGrid testGrid = IRObject as IRGrid;
            MPGrid targetGrid = HitObject as MPGrid;
            Point targetPoint = new Point();
            
            targetPoint = testGrid.TranslatePoint(new Point(0, 0), targetGrid);

            if (targetPoint.X > 0 &&
                targetPoint.Y > 0 &&
                targetPoint.X < targetGrid.ActualWidth &&
                targetPoint.Y < targetGrid.ActualHeight)
            {
                return true;
            }

            else
            {
                return false;
            }
        }

        public bool isMPGridPinched()
        {
            return false;
        }

    }

    public class IRGrid : Grid
    {
       static IRGrid()
        {
            IRGrid.VelocityProperty = DependencyProperty.Register("Velocity",
                        typeof(Point), typeof(IRGrid),
                        new FrameworkPropertyMetadata(new Point(0,0)));

            IRGrid.LastPositionProperty = DependencyProperty.Register("LastPosition",
                        typeof(Point), typeof(IRGrid),
                        new FrameworkPropertyMetadata(new Point(0, 0)));

            IRGrid.CurrentPositionProperty = DependencyProperty.Register("CurrentPosition",
                         typeof(Point), typeof(IRGrid),
                         new FrameworkPropertyMetadata(new Point(0, 0), new PropertyChangedCallback(setLastPosition)));

           IRGrid.IsLeftHandProperty = DependencyProperty.Register("IsLeftHand",
                        typeof(bool), typeof(IRGrid),
                        new FrameworkPropertyMetadata(false));

           IRGrid.IsRightHandProperty = DependencyProperty.Register("IsRightHand",
                        typeof(bool), typeof(IRGrid),
                        new FrameworkPropertyMetadata(false));

           IRGrid.IsIRSeenProperty = DependencyProperty.Register("IsIRSeen",
                        typeof(bool), typeof(IRGrid));

           IRGrid.IsPinchedProperty = DependencyProperty.Register("IsPinched",
                        typeof(bool), typeof(IRGrid));
        }
       
        public Point Velocity
        {
            get { return (Point)GetValue(IRGrid.VelocityProperty); }
            set { SetValue(IRGrid.VelocityProperty, value); }
        }

        public Point LastPosition
        {
            get { return (Point)GetValue(IRGrid.LastPositionProperty); }
            set { SetValue(IRGrid.LastPositionProperty, value); }
        }

        public Point CurrentPosition
        {
            get { return (Point)GetValue(IRGrid.CurrentPositionProperty); }
            set { SetValue(IRGrid.CurrentPositionProperty, value); }
        }

        public bool IsLeftHand
        {
            get { return (bool)GetValue(IRGrid.IsLeftHandProperty); }
            set { SetValue(IRGrid.IsLeftHandProperty, value); }
        }

        public bool IsRightHand
        {
            get { return (bool)GetValue(IRGrid.IsRightHandProperty); }
            set { SetValue(IRGrid.IsRightHandProperty, value); }
        }

        public bool IsIRSeen
        {
            get { return (bool)GetValue(IRGrid.IsIRSeenProperty); }
            set { SetValue(IRGrid.IsIRSeenProperty, value); }
        }

        public bool IsPinched
        {
            get { return (bool)GetValue(IRGrid.IsPinchedProperty); }
            set { SetValue(IRGrid.IsPinchedProperty, value); }
        }

        private static void setLastPosition(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
            /*IRGrid holder = o as IRGrid;
            Point OldPoint = holder.LastPosition;
            Point NewPoint = (Point)e.NewValue;
            Point velocityPoint = new Point((NewPoint.X - OldPoint.X), (NewPoint.Y - OldPoint.Y));
            holder.Velocity = velocityPoint;
            holder.LastPosition = NewPoint;            */
        }

        public static DependencyProperty LastPositionProperty;
        public static DependencyProperty CurrentPositionProperty;
        public static DependencyProperty VelocityProperty;

        public static DependencyProperty IsLeftHandProperty;
        public static DependencyProperty IsRightHandProperty;
        public static DependencyProperty IsPinchedProperty;
        public static DependencyProperty IsIRSeenProperty;

        

    }

 
}